use crate::contexts;
use crate::controllers::posts;
use crate::models::NewPost;
use rocket::form::Form;
use rocket::http::{Cookie, CookieJar};
use rocket::response::Redirect;
use rocket::Route;
use rocket_dyn_templates::Template;
use std::env;
use std::vec::Vec;

#[derive(FromForm)]
struct Login {
    username: String,
    password: String,
}

#[get("/")]
fn index(cookies: &CookieJar<'_>) -> Result<Redirect, Template> {
    let password = env::var("admin_pass").expect("admin_pass not setted");
    match cookies.get_private("user") {
        Some(cookie) => {
            if cookie.value() == password {
                Ok(Redirect::to("/admin/panel"))
            } else {
                let context = contexts::Empty::new();

                Err(Template::render("admin/index", context))
            }
        }
        None => {
            let context = contexts::Empty::new();

            Err(Template::render("admin/index", context))
        }
    }
}

#[post("/", data = "<login>")]
fn login(cookies: &CookieJar<'_>, login: Form<Login>) -> Redirect {
    let username = env::var("admin_user").expect("admin_user not setted");
    let password = env::var("admin_pass").expect("admin_pass not setted");

    if login.username == username && login.password == password {
        cookies.add_private(Cookie::new("user", password));
        Redirect::to("/admin/panel")
    } else {
        Redirect::to("/admin")
    }
}

#[get("/panel")]
fn panel(cookies: &CookieJar<'_>) -> Result<Template, Redirect> {
    let password = env::var("admin_pass").expect("admin_pass not setted");

    match cookies.get_private("user") {
        Some(cookie) => {
            if cookie.value() == password {
                let (all_posts, n_posts) = posts::get_posts(None);

                let context = contexts::AllPosts::new(all_posts, n_posts);

                Ok(Template::render("admin/panel", context))
            } else {
                Err(Redirect::to("/admin"))
            }
        }
        None => Err(Redirect::to("/admin")),
    }
}

#[get("/add_post")]
fn add_post(cookies: &CookieJar<'_>) -> Result<Template, Redirect> {
    let password = env::var("admin_pass").expect("admin_pass not setted");

    match cookies.get_private("user") {
        Some(cookie) => {
            if cookie.value() == password {
                let context = contexts::Empty::new();

                Ok(Template::render("admin/add_post", context))
            } else {
                Err(Redirect::to("/admin"))
            }
        }
        None => Err(Redirect::to("/admin")),
    }
}

#[post("/add_post", data = "<post>")]
fn write_add_post(cookies: &CookieJar<'_>, post: Form<NewPost>) -> Redirect {
    let password = env::var("admin_pass").expect("admin_pass not setted");

    match cookies.get_private("user") {
        Some(cookie) => {
            if cookie.value() == password {
                match posts::add_post(&post) {
                    Ok(post) => Redirect::to(format!("/post/{}", post.id)),
                    Err(_) => Redirect::to("admin/add_post"),
                }
            } else {
                Redirect::to("/admin")
            }
        }
        None => Redirect::to("/admin"),
    }
}

#[get("/edit_post/<id>")]
fn edit_post(cookies: &CookieJar<'_>, id: i32) -> Result<Template, Redirect> {
    let password = env::var("admin_pass").expect("admin_pass not setted");

    match cookies.get_private("user") {
        Some(cookie) => {
            if cookie.value() == password {
                let post = posts::get_post(id).unwrap();

                let context = contexts::SinglePost::new(post);

                Ok(Template::render("admin/edit_post", context))
            } else {
                Err(Redirect::to("/admin"))
            }
        }
        None => Err(Redirect::to("/admin")),
    }
}

#[post("/edit_post/<id>", data = "<post>")]
fn write_edit_post(cookies: &CookieJar<'_>, id: i32, post: Form<NewPost>) -> Redirect {
    let password = env::var("admin_pass").expect("admin_pass not setted");

    match cookies.get_private("user") {
        Some(cookie) => {
            if cookie.value() == password {
                match posts::edit_post(id, &post) {
                    Ok(post) => Redirect::to(format!("/post/{}", post.id)),
                    Err(_) => Redirect::to("admin/edit_post"),
                }
            } else {
                Redirect::to("/admin")
            }
        }
        None => Redirect::to("/admin"),
    }
}

pub fn get_routes() -> Vec<Route> {
    routes![
        index,
        login,
        panel,
        add_post,
        write_add_post,
        edit_post,
        write_edit_post
    ]
}
